/**************************************************************

File Name:		Complex_Block_FIR_test.asm
		
Date Modified:	01/05/99	RFG
				05/01/00	RFG
				12/20/04    BJM - Ported to ADSP-21364

Description:	Program to call Complex_Block_FIR.asm

**************************************************************/

#include    <def21364.h>				/* Symbol Definition File */

#define     TAPS    16					/* length of filter */
#define     N       40					/* number of samples */


.extern complex_block_fir;

/* DM data */
.section/dm seg_dmda;					/* Segments are declared in the .ldf file*/
.ALIGN 2;								/* Set alignment to long-word boundary for next variable */
.var    dline[2*TAPS];					/* delay line of complex (interleaved) input data */
.ALIGN 2;
.var    input[2*N] = "indata.dat";		/* input array of complex (interleaved: real[0],imag[0],real[1],imag[1] samples for input data */


/* PM long-word data */
.section/pm seg_pm64;
.ALIGN 2;
// View interleaved output at 0x9FE02
.var    output[2*N+2];  				/* the extra word at the beginning of the output buffer allows an optimization in the outer */
                        				/* loop of the kernel -- the write of the previously computed real/imag output samples is */
										/* done at the top of the outer loop rather than at the bottom. */
                        				/* this allows a dual parallel move and saves 1 cycle */

/* PM data */
.section/dm seg_pmda;
.ALIGN 2;
.var    coeffs[2*TAPS] = "coeffs.dat";	/* complex (interleaved) coeffiecient array */


/* PM interrupt vector code */
.section/pm   seg_rth;
Reserved_1:     rti; nop; nop; nop;
Chip_Reset:     idle; jump start; nop; nop;


/* program memory code */
.section/pm seg_pmco;

start:

/*--------------------- Setup modify registers for arrays --------------------*/
	m1 = 1;
    m2 = 2; 
	m3 = -1;
	m4 = -2;
	  
	m9 = 2;
	m10 = 1;
/*----------------------------------------------------------------------------*/



/*--------------------- Initialization delay line ----------------------------*/
	b0 = dline;
	l0 = @dline;

	r0 = b0;
	r0 = lshift r0 by -1;
	  
	b1 = r0;                /* Set a pointer to the delay line in Long-word space */
	l1 = @dline/2;          /* to do a dual data, SIMD, Long-word access. */

	bit set MODE1 RND32 | PEYEN | CBUFEN;
	
	lcntr = TAPS, do clear_fir until lce;
clear_fir:  dm(i1,m2) = 0.0;

	i1 = r0;
/*----------------------------------------------------------------------------*/



/*------------------------- Setup DAGs for coeffs ----------------------------*/
	b8 = coeffs;
	l8 = @coeffs;
/*----------------------------------------------------------------------------*/



/*-------------- Setup DAGs for input/output and call cx_block_fir -----------*/
	b2 = input;
	l2 = 0;

	b9 = output;
	l9 = 0;

	call complex_block_fir (db);
	r2 = N;
	r3 = (TAPS-4)/4;
/*----------------------------------------------------------------------------*/


	  
/* Terminate and wait */
wait1:	idle;
		jump wait1;
